#
# Copyright (c) Daniel Sheffield 2021 - 2022
#
# All rights reserved
#
# THIS SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY
from collections import OrderedDict
from typing import Callable, List, Union

import urwid
from app.activities import ActivityManager
from app.db_utils import QueryManager

from .. import COPYRIGHT
from ..widgets import (AutoCompleteEdit, AutoCompleteFloatEdit,
                       AutoCompletePopUp, FocusWidget, NoTabCheckBox)


class TransactionEditor(FocusWidget):

    def keypress(self, size, key):
        if isinstance(key, tuple):
            return
        
        if getattr(self.original_widget.original_widget, 'original_widget', None) is None:
            return super().keypress(size, key)
        
        if key == 'tab':
            self.advance_focus()
        elif key == 'shift tab':
            self.advance_focus(reverse=True)
        elif key == 'ctrl delete':
            self.clear()
            self.focus_on_product()
        elif key == 'insert':
            self.save_and_clear_cb()
        elif key == 'ctrl p':
            self.focus_on_product()
        else:
            return super().keypress(size, key)

    def _init_data(self, fields: List[str]) -> None:
        self.data = OrderedDict([
            (k, '') for k in fields
        ])
        self.clear()

    def apply_choice(self, name, value):
        self.apply_changes(name, value)
        for k,v in self.data.items():
            if k == name or v:
                continue
            options = self.query_manager.unique_suggestions(k, **self.data)
            if len(options) == 1 and k != 'ts':
                self.apply_changes(k, list(options)[0])

    def apply_changes(self, name, value):
        self.data.update({
            name: value if name != 'organic' else {
                'yes': True, 'no': False,
                True: True, False: False,
                'mixed': False
            }[value],
        })

    def apply_organic_state(self, w, state):
        self.data['organic'] = state if state != 'mixed' else False

    def clear(self):
        for k in self.data:
            if k in ('ts', 'store',):
                continue
            if k == 'organic':
                self.data[k] = False
                continue
            self.data[k] = ''

        return self.update()
    
    def update(self):
        for k in self.edit_fields:
            self.edit_fields[k].set_edit_text(self.data[k])
        date, store = self.data['ts'], self.data['store']
        self.text_fields['dbview'].set_text(
            self.query_manager.get_session_transactions(date, store) if None not in (
                date or None, store or None
            ) else ''
        )
        self.organic_checkbox.set_state(self.data['organic'])
        return self

    def focus_on_product(self):
        path = self.container.get_focus_path()
        while path != [2,0,1,0,0]:
            self.advance_focus()
            path = self.container.get_focus_path()

    def __init__(self,
        activity_manager: ActivityManager,
        query_manager: QueryManager,
        fields: List[str],
        layout, side_pane, bottom_pane,
        save_and_clear_cb,
        autocomplete_cb: Callable[[
            Union[AutoCompleteEdit, AutoCompleteFloatEdit], str, dict
        ], None],
    ):
        self.organic_checkbox = NoTabCheckBox(
            u"Organic",
            on_state_change=self.apply_organic_state
        )
        self.query_manager = query_manager
        self.save_and_clear_cb = save_and_clear_cb
        self.layout = layout
        self.side_pane = side_pane
        self.bottom_pane = bottom_pane
        self.edit_fields = OrderedDict()
        self.text_fields = OrderedDict()
        widgets = dict()
        txn_view = urwid.Text('')
        self.text_fields.update({'dbview': txn_view})
        widgets.update({
            'dbview': urwid.LineBox(
                urwid.AttrMap(txn_view, 'streak'),
                title="Session Data",
                title_align='left',
            )
        })
        self._init_data(fields)
        for k in self.data:
            if k in self.side_pane and k != 'unit':
                ef = AutoCompleteFloatEdit(('bg', k))
            elif k != 'organic':
                ef = AutoCompleteEdit(('bg', k))
            else:
                continue

            ef.set_edit_text(self.data[k])
            urwid.connect_signal(ef, 'change', lambda w, v: self.apply_changes(w.name, v))
            urwid.connect_signal(ef, 'apply', lambda w, name: autocomplete_cb(w, name, self.data))
            self.edit_fields[k] = ef
    
        widgets.update(dict([
            (k, urwid.LineBox(
                urwid.AttrMap(AutoCompletePopUp(
                    self.edit_fields[k],
                    self.apply_choice,
                    lambda: activity_manager.show(self.update())
                ), 'streak'), title=k.title(), title_align='left')
            ) for k in self.edit_fields if k != 'product'
        ]))
        header = urwid.Text(u'Fill Transaction', 'center')
        _copyright = urwid.Text(COPYRIGHT, 'center')
        done_button = urwid.Button(('streak', u'Done'))
        clear_button = urwid.Button(('streak', u'Clear'))
        urwid.connect_signal(done_button, 'click', lambda _: save_and_clear_cb())
        urwid.connect_signal(clear_button, 'click', lambda _: self.clear())
        banner = urwid.Pile([
            urwid.Padding(header, 'center', width=('relative', 100)),
            urwid.Padding(_copyright, 'center', width=('relative', 100)),
        ])
        banner = urwid.AttrMap(banner, 'banner')
        widgets.update({
            'product': urwid.LineBox(urwid.Columns([
                urwid.AttrMap(AutoCompletePopUp(
                    self.edit_fields['product'],
                    self.apply_choice,
                    lambda: activity_manager.show(self.update())
                ), 'streak'),
                self.organic_checkbox
            ], dividechars=2), title='Product', title_align='left')
        })
        
        side_pane_widget = (12, urwid.Pile([
            widgets[r] if r is not None else urwid.Divider() for r in self.side_pane
        ]))
        main_pane_widgets = []
        for i, r in enumerate(self.layout):
            _widgets = []
            for c in r:
                if c is not None:
                    if c == 'organic':
                        continue
                    _widgets.append(widgets[c])
                else:
                    _widgets.append(urwid.Divider())
            main_pane_widgets.append(urwid.Columns(_widgets))


        main_pane_widget = urwid.Pile(main_pane_widgets)
        
        widget = urwid.Pile([
            banner,
            urwid.Divider(),
            urwid.Columns([
                    main_pane_widget, side_pane_widget
                ],
                dividechars=2,
            ),
            *[ widgets[c] if c is not None else urwid.Divider() for c in self.bottom_pane ],
            urwid.Divider(),
            urwid.Columns([
                (8, done_button), urwid.Divider(), (9, clear_button)
            ])
        ])
        widget = urwid.Filler(widget, 'top') 
        widget = urwid.AttrMap(widget, 'bg')
        super().__init__(widget, 5, [2,0,0,0], [
            [0, 0], [0,1],
            [1,],
            [2,], [2,0], [2,0,0],
            [2, 0, 1, 0],
            [4,],
            [5,],
            [6,],
            [6, 1],
            [7,],
        ])
